@@ -24,6 +24,8 @@
 #include "msdos.h"
 #include "stream.h"
 #include "mtools.h"
+#include "device.h"
+#include "old_dos.h"
 #include "fsP.h"
 #include "buffer.h"
 #include "file_name.h"
@@ -31,6 +33,11 @@
 
 #define FULL_CYL
 
+mt_off_t sectorsToBytes(Fs_t *This, uint32_t off)
+{
+	return (mt_off_t) off << This->sectorShift;
+}
+
 /*
  * Read the boot sector.  We glean the disk parameters from this sector.
  */
@@ -44,10 +51,10 @@ static int read_boot(Stream_t *Stream, union bootsector * boot, size_t size)
 	if(size > MAX_BOOT)
 		size = MAX_BOOT;
 
-	if (force_read(Stream, boot->characters, 0, size) != (ssize_t) size)
+	if (force_pread(Stream, boot->characters, 0, size) != (ssize_t) size)
 		return -1;
 
-	boot_sector_size = WORD(secsiz);		
+	boot_sector_size = WORD(secsiz);
 	if(boot_sector_size < sizeof(boot->bytes)) {
 		/* zero rest of in-memory boot sector */
 		memset(boot->bytes+boot_sector_size, 0,
@@ -72,8 +79,10 @@ static doscp_t *get_dosConvert(Stream_t *Stream)
 }
 
 Class_t FsClass = {
-	read_pass_through, /* read */
-	write_pass_through, /* write */
+	0,
+	0,
+	pread_pass_through, /* read */
+	pwrite_pass_through, /* write */
 	fs_flush,
 	fs_free, /* free */
 	0, /* set geometry */
@@ -97,7 +106,7 @@ static int get_media_type(Stream_t *St, union bootsector *boot)
 		char temp[512];
 		/* old DOS disk. Media descriptor in the first FAT byte */
 		/* we assume 512-byte sectors here */
-		if (force_read(St,temp,(mt_off_t) 512,512) == 512)
+		if (force_pread(St,temp,512,512) == 512)
 			media = (unsigned char) temp[0];
 		else
 			media = 0;
@@ -126,7 +135,7 @@ static void boot_to_geom(struct device *dev, int media,
 	dev->ssize = 2; /* allow for init_geom to change it */
 	dev->use_2m = 0x80; /* disable 2m mode to begin */
 
-	if(media == 0xf0 || media >= 0x100){		
+	if(media == 0xf0 || media >= 0x100){
 		dev->heads = WORD(nheads);
 		dev->sectors = WORD(nsect);
 		tot_sectors = DWORD(bigsect);
@@ -152,12 +161,12 @@ static void boot_to_geom(struct device *dev, int media,
 		if(tot_sectors % sect_per_track)
 			/* round size up */
 			dev->tracks++;
-		
+
 		BootP = WORD(ext.old.BootP);
 		Infp0 = WORD(ext.old.Infp0);
 		InfpX = WORD(ext.old.InfpX);
 		InfTm = WORD(ext.old.InfTm);
-		
+
 		if(WORD(fatlen)) {
 			labelBlock = &boot->boot.ext.old.labelBlock;
 		} else {
@@ -168,9 +177,9 @@ static void boot_to_geom(struct device *dev, int media,
 		    has_BPB4 &&
 		    strncmp( boot->boot.banner,"2M", 2 ) == 0 &&
 		    BootP < 512 && Infp0 < 512 && InfpX < 512 && InfTm < 512 &&
-		    BootP >= InfTm + 2 && InfTm >= InfpX && InfpX >= Infp0 && 
+		    BootP >= InfTm + 2 && InfTm >= InfpX && InfpX >= Infp0 &&
 		    Infp0 >= 76 ){
-			for (sum=0, j=63; j < BootP; j++) 
+			for (sum=0, j=63; j < BootP; j++)
 				sum += boot->bytes[j];/* checksum */
 			dev->ssize = boot->bytes[InfTm];
 			if (!sum && dev->ssize <= 7){
@@ -208,7 +217,7 @@ static void boot_to_geom(struct device *dev, int media,
 static Stream_t *try_device(struct device *dev,
 			    int mode, struct device *out_dev,
 			    union bootsector *boot,
-			    char *name, int *media, mt_size_t *maxSize,
+			    char *name, int *media, mt_off_t *maxSize,
 			    int *isRop, int try_writable,
 			    char *errmsg)
 {
@@ -247,7 +256,7 @@ static Stream_t *try_device(struct device *dev,
 		if(Stream == NULL) {
 			if(geomFailure && (mode & O_ACCMODE) == O_RDONLY) {
 				/* Our first attempt was to open read-only,
-				   but this resulted in failure setting the 
+				   but this resulted in failure setting the
 				   geometry */
 				openMode = modeFlags | O_RDWR;
 				continue;
@@ -323,6 +332,22 @@ static Stream_t *try_device(struct device *dev,
 	return NULL;
 }
 
+uint32_t calc_clus_start(Fs_t *Fs) {
+	return Fs->fat_start + Fs->fat_len*Fs->num_fat + Fs->dir_len;
+}
+
+/* Calculates number of clusters, and fills it in into Fs->num_clus
+ * Returns 0 if calculation could be performed, and -1 if less sectors than
+ * clus_start
+ */
+int calc_num_clus(Fs_t *Fs, uint32_t tot_sectors)
+{
+	Fs->clus_start = calc_clus_start(Fs);
+	if(tot_sectors <= Fs->clus_start)
+		return -1;
+	Fs->num_clus = (tot_sectors - Fs->clus_start) / Fs->cluster_size;
+	return 0;
+}
 
 /**
  * Tries out all device definitions for the given drive letter, until one
@@ -342,12 +367,12 @@ static Stream_t *try_device(struct device *dev,
  */
 Stream_t *find_device(char drive, int mode, struct device *out_dev,
 		      union bootsector *boot,
-		      char *name, int *media, mt_size_t *maxSize,
+		      char *name, int *media, mt_off_t *maxSize,
 		      int *isRop)
 {
 	char errmsg[200];
 	struct device *dev;
-	
+
 	sprintf(errmsg, "Drive '%c:' not supported", drive);
 					/* open the device */
 	for (dev=devices; dev->name; dev++) {
@@ -375,44 +400,13 @@ Stream_t *find_device(char drive, int mode, struct device *out_dev,
 }
 
 
-Stream_t *fs_init(char drive, int mode, int *isRop)
+uint32_t parseFsParams(	Fs_t *This,
+			union bootsector *boot,
+			int media,
+			unsigned int cylinder_size)
 {
-	uint32_t blocksize;
-	int media;
-	size_t disk_size = 0;	/* In case we don't happen to set this below */
 	uint32_t tot_sectors;
-	char name[EXPAND_BUF];
-	unsigned int cylinder_size;
-	struct device dev;
-	mt_size_t maxSize;
 
-	union bootsector boot;
-
-	Fs_t *This;
-
-	This = New(Fs_t);
-	if (!This)
-		return NULL;
-
-	This->Direct = NULL;
-	This->Next = NULL;
-	This->refs = 1;
-	This->Buffer = 0;
-	This->Class = &FsClass;
-	This->preallocatedClusters = 0;
-	This->lastFatSectorNr = 0;
-	This->lastFatAccessMode = 0;
-	This->lastFatSectorData = 0;
-	This->drive = drive;
-	This->last = 0;
-
-	This->Direct = find_device(drive, mode, &dev, &boot, name, &media,
-				   &maxSize, isRop);
-	if(!This->Direct)
-		return NULL;
-
-	cylinder_size = dev.heads * dev.sectors;
-	This->serialized = 0;
 	if ((media & ~7) == 0xf8){
 		/* This bit of code is only entered if there is no BPB, or
 		 * else result of the AND would be 0x1xx
@@ -420,7 +414,7 @@ Stream_t *fs_init(char drive, int mode, int *isRop)
 		struct OldDos_t *params=getOldDosByMedia(media);
 		if(params == NULL) {
 			fprintf(stderr, "Unknown media byte %02x\n", media);
-			return NULL;
+			return 0;
 		}
 		This->cluster_size = params->cluster_size;
 		tot_sectors = cylinder_size * params->tracks;
@@ -431,24 +425,23 @@ Stream_t *fs_init(char drive, int mode, int *isRop)
 		This->sector_size = 512;
 		This->sectorShift = 9;
 		This->sectorMask = 511;
-		This->fat_bits = 12;
 	} else {
 		struct label_blk_t *labelBlock;
 		unsigned int i;
-		
-		This->sector_size = WORD_S(secsiz);
+
+		This->sector_size = WORD(secsiz);
 		if(This->sector_size > MAX_SECTOR){
-			fprintf(stderr,"init %c: sector size too big\n", drive);
-			return NULL;
+			fprintf(stderr,"init: sector size too big\n");
+			return 0;
 		}
 
 		i = log_2(This->sector_size);
 
 		if(i == 24) {
 			fprintf(stderr,
-				"init %c: sector size (%d) not a small power of two\n",
-				drive, This->sector_size);
-			return NULL;
+				"init: sector size (%d) not a small power of two\n",
+				This->sector_size);
+			return 0;
 		}
 		This->sectorShift = i;
 		This->sectorMask = This->sector_size - 1;
@@ -457,20 +450,22 @@ Stream_t *fs_init(char drive, int mode, int *isRop)
 		 * all numbers are in sectors, except num_clus
 		 * (which is in clusters)
 		 */
-		tot_sectors = WORD_S(psect);
+		tot_sectors = WORD(psect);
 		if(!tot_sectors)
-			tot_sectors = DWORD_S(bigsect);	
+			tot_sectors = DWORD(bigsect);
 
-		This->cluster_size = boot.boot.clsiz;
-		This->fat_start = WORD_S(nrsvsect);
-		This->fat_len = WORD_S(fatlen);
-		This->dir_len = WORD_S(dirents) * MDIR_SIZE / This->sector_size;
-		This->num_fat = boot.boot.nfat;
+		This->cluster_size = boot->boot.clsiz;
+		This->fat_start = WORD(nrsvsect);
+		This->fat_len = WORD(fatlen);
+		This->dir_len = WORD(dirents) * MDIR_SIZE / This->sector_size;
+		This->num_fat = boot->boot.nfat;
 
 		if (This->fat_len) {
-			labelBlock = &boot.boot.ext.old.labelBlock;
+			labelBlock = &boot->boot.ext.old.labelBlock;
 		} else {
-			labelBlock = &boot.boot.ext.fat32.labelBlock;
+			labelBlock = &boot->boot.ext.fat32.labelBlock;
+			This->fat_len = DWORD(ext.fat32.bigFat);
+			This->backupBoot = WORD(ext.fat32.backupBoot);
 		}
 
 		if(has_BPB4) {
@@ -479,9 +474,61 @@ Stream_t *fs_init(char drive, int mode, int *isRop)
 		}
 	}
 
-	if (tot_sectors >= (maxSize >> This->sectorShift)) {
-		fprintf(stderr, "Big disks not supported on this architecture\n");
-		exit(1);
+	if(calc_num_clus(This, tot_sectors) < 0)
+		/* Too few sectors */
+		return 0;
+	set_fat(This);
+
+	return tot_sectors;
+}
+
+
+Stream_t *fs_init(char drive, int mode, int *isRop)
+{
+	uint32_t blocksize;
+	int media;
+	size_t disk_size = 0;	/* In case we don't happen to set this below */
+	uint32_t tot_sectors;
+	char name[EXPAND_BUF];
+	unsigned int cylinder_size;
+	struct device dev;
+	mt_off_t maxSize;
+	char errmsg[81];
+	
+	union bootsector boot;
+
+	Fs_t *This;
+
+	This = New(Fs_t);
+	if (!This)
+		return NULL;
+
+	init_head(&This->head, &FsClass, NULL);
+	This->preallocatedClusters = 0;
+	This->lastFatSectorNr = 0;
+	This->lastFatAccessMode = 0;
+	This->lastFatSectorData = 0;
+	This->drive = drive;
+	This->last = 0;
+
+	This->head.Next = find_device(drive, mode, &dev, &boot, name, &media,
+				      &maxSize, isRop);
+	if(!This->head.Next)
+		return NULL;
+
+	cylinder_size = dev.heads * dev.sectors;
+	This->serialized = 0;
+
+	tot_sectors = parseFsParams(This, &boot, media, cylinder_size);
+	if(tot_sectors == 0) {
+		/* Error raised by parseFsParams */
+		return NULL;
+	}
+
+	if (check_if_sectors_fit(tot_sectors, maxSize,
+				 This->sector_size, errmsg) < 0) {
+		fprintf(stderr, "%s", errmsg);
+		return NULL;
 	}
 
 	/* full cylinder buffering */
@@ -516,25 +563,24 @@ Stream_t *fs_init(char drive, int mode, int *isRop)
 		blocksize = This->sector_size;
 	else
 		blocksize = dev.blocksize;
-	if (disk_size)
-		This->Next = buf_init(This->Direct,
-				      8 * disk_size * blocksize,
-				      disk_size * blocksize,
-				      This->sector_size);
-	else
-		This->Next = This->Direct;
+	if (disk_size) {
+		Stream_t *Buffer = buf_init(This->head.Next,
+					    8 * disk_size * blocksize,
+					    disk_size * blocksize,
+					    This->sector_size);
 
-	if (This->Next == NULL) {
-		perror("init: allocate buffer");
-		This->Next = This->Direct;
+		if (Buffer != NULL)
+			This->head.Next = Buffer;
+		else
+			perror("init: allocate buffer");
 	}
 
 	/* read the FAT sectors */
-	if(fat_read(This, &boot, tot_sectors, dev.use_2m&0x7f)){
+	if(fat_read(This, &boot, dev.use_2m&0x7f)){
 		fprintf(stderr, "Error reading FAT\n");
 		This->num_fat = 1;
-		FREE(&This->Next);
-		Free(This->Next);
+		FREE(&This->head.Next);
+		Free(This->head.Next);
 		return NULL;
 	}
 
@@ -543,8 +589,8 @@ Stream_t *fs_init(char drive, int mode, int *isRop)
 	if(This->cp == NULL) {
 		fprintf(stderr, "Error setting code page\n");
 		fs_free((Stream_t *)This);
-		FREE(&This->Next);
-		Free(This->Next);
+		FREE(&This->head.Next);
+		Free(This->head.Next);
 		return NULL;
 	}
 
@@ -555,7 +601,7 @@ char getDrive(Stream_t *Stream)
 {
 	DeclareThis(Fs_t);
 
-	if(This->Class != &FsClass)
+	if(This->head.Class != &FsClass)
 		return getDrive(GetFs(Stream));
 	else
 		return This->drive;
